@@ -9,9 +9,6 @@
 *
 ***********************************************************************/
 
-static char const RCSID[] =
-"$Id$";
-
 #include "event_tcp.h"
 #include <unistd.h>
 #include <fcntl.h>
@@ -65,6 +62,13 @@ handle_accept(EventSelector *es,
 	if (conn < 0) return;
 	/* Set close-on-exec flag */
 	set_cloexec(conn);
+
+	/* Make conn non-blocking */
+	if (set_nonblocking(conn) < 0) {
+	    close(conn);
+	    return;
+	}
+
 	f(es, conn);
     }
 }
@@ -158,15 +162,9 @@ EventTcp_CreateAcceptor(EventSelector *es,
 			int socket,
 			EventTcpAcceptFunc f)
 {
-    int flags;
-
     EVENT_DEBUG(("EventTcp_CreateAcceptor(es=%p, socket=%d)\n", es, socket));
-    /* Make sure socket is non-blocking */
-    flags = fcntl(socket, F_GETFL, 0);
-    if (flags == -1) {
-	return NULL;
-    }
-    if (fcntl(socket, F_SETFL, flags | O_NONBLOCK) == -1) {
+
+    if (set_nonblocking(socket) < 0) {
 	return NULL;
     }
 
@@ -575,22 +573,12 @@ EventTcp_ReadBuf(EventSelector *es,
 		 void *data)
 {
     EventTcpState *state;
-    int flags;
     struct timeval t;
 
     EVENT_DEBUG(("EventTcp_ReadBuf(es=%p, socket=%d, len=%d, delim=%d, timeout=%d)\n", es, socket, len, delim, timeout));
     if (len <= 0) return NULL;
     if (socket < 0) return NULL;
 
-    /* Make sure socket is non-blocking */
-    flags = fcntl(socket, F_GETFL, 0);
-    if (flags == -1) {
-	return NULL;
-    }
-    if (fcntl(socket, F_SETFL, flags | O_NONBLOCK) == -1) {
-	return NULL;
-    }
-
     state = malloc(sizeof(EventTcpState));
     if (!state) return NULL;
     memset(state, 0, sizeof(EventTcpState));
@@ -655,7 +643,6 @@ EventTcp_WriteBuf(EventSelector *es,
 		  void *data)
 {
     EventTcpState *state;
-    int flags;
     struct timeval t;
 
     EVENT_DEBUG(("EventTcp_WriteBuf(es=%p, socket=%d, len=%d, timeout=%d)\n", es, socket, len, timeout));
@@ -665,17 +652,6 @@ EventTcp_WriteBuf(EventSelector *es,
 	return NULL;
     }
 
-    /* Make sure socket is non-blocking */
-    flags = fcntl(socket, F_GETFL, 0);
-    if (flags == -1) {
-	if (!f) close(socket);
-	return NULL;
-    }
-    if (fcntl(socket, F_SETFL, flags | O_NONBLOCK) == -1) {
-	if (!f) close(socket);
-	return NULL;
-    }
-
     state = malloc(sizeof(EventTcpState));
     if (!state) {
 	if (!f) close(socket);
@@ -744,14 +720,12 @@ EventTcp_Connect(EventSelector *es,
 		 int timeout,
 		 void *data)
 {
-    int flags;
     int n;
     EventTcpConnectState *state;
     struct timeval t;
 
     /* Make sure socket is non-blocking */
-    flags = fcntl(fd, F_GETFL, 0);
-    if (flags == -1 || fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
+    if (set_nonblocking(fd) < 0) {
 	f(es, fd, EVENT_TCP_FLAG_IOERROR, data);
 	return;
     }
