@@ -151,11 +151,22 @@ static char *EquivToLoopback = NULL;
 static char ValidateHeader[256];
 
 /* Additional Sendmail macros to pass along */
-/// TODO Make the limit 256
-#define MAX_ADDITIONAL_SENDMAIL_MACROS 32
+#define MAX_ADDITIONAL_SENDMAIL_MACROS 256
 static char *AdditionalMacros[MAX_ADDITIONAL_SENDMAIL_MACROS];
 static int NumAdditionalMacros = 0;
 
+/* Standard Sendmail macros */
+/* We can't make it char const * because libmilter
+   is not const-correct. */
+static char *StandardSendmailMacros[] = {
+    "_", "auth_authen", "auth_author", "auth_ssf", "auth_type",
+    "cert_issuer", "cert_subject", "cipher", "cipher_bits", "daemon_name",
+    "i", "if_addr", "if_name", "j", "mail_addr", "mail_host", "mail_mailer",
+    "tls_version", "verify",
+    /* End of macros MUST be marked with NULL! */
+    NULL
+};
+
 /* Keep track of private data -- file name and fp for writing e-mail body */
 struct privdata {
     char *hostname;		/* Name of connecting host */
@@ -166,6 +177,7 @@ struct privdata {
     char *dir;			/* Work directory */
     char *heloArg;		/* HELO argument */
     char *qid;                  /* Queue ID */
+    unsigned char qid_written;  /* Have we written qid to COMMANDS? */
     int fd;			/* File for message body */
     int headerFD;		/* File for message headers */
     int cmdFD;			/* File for commands */
@@ -543,6 +555,7 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
     data->firstRecip = NULL;
     data->dir      = NULL;
     data->heloArg  = NULL;
+    data->qid_written = 0;
     data->qid      = NOQUEUE;
     data->fd       = -1;
     data->headerFD = -1;
@@ -768,11 +781,16 @@ envfrom(SMFICTX *ctx, char **from)
     char buf2[SMALLBUF];
     char *queueid;
     char *me;
+    char **macro;
     dynamic_buffer dbuf;
 
     char mxid[MX_ID_LEN+1];
 
     DEBUG_ENTER("envfrom");
+
+    /* Generate the MIMEDefang ID */
+    (void) gen_mx_id(mxid);
+
     /* Get the private context */
     data = DATA;
     if (!data) {
@@ -815,16 +833,15 @@ envfrom(SMFICTX *ctx, char **from)
 	data->firstRecip = NULL;
     }
 
-    /* Try making a directory whose name contains the Sendmail qid first */
+    /* Make the working directory */
     if (!data->qid || data->qid == NOQUEUE) {
-        // We have no queue ID, so fail.
-	syslog(LOG_WARNING, "No queue ID available in envfrom()");
-	cleanup(ctx);
-	DEBUG_EXIT("envfrom", "SMFIS_TEMPFAIL");
-	return SMFIS_TEMPFAIL;
+	/* No queue ID, so use the mxid */
+	snprintf(buffer, SMALLBUF, "%s/mdefang-%s", SpoolDir, mxid);
+    } else {
+	/* We have a queue ID, so use it */
+	snprintf(buffer, SMALLBUF, "%s/mdefang-%s", SpoolDir, data->qid);
     }
 
-    snprintf(buffer, SMALLBUF, "%s/mdefang-%s", SpoolDir, data->qid);
     if (mkdir(buffer, 0750) != 0) {
         /* Could not create temp. directory */
 	syslog(LOG_WARNING, "%s: Could not create directory %s: %m",
@@ -834,19 +851,24 @@ envfrom(SMFICTX *ctx, char **from)
 	return SMFIS_TEMPFAIL;
     }
 
-    if (data->dir) free(data->dir);
     if (data->fd >= 0) closefd(data->fd);
     if (data->headerFD >= 0) closefd(data->headerFD);
     if (data->cmdFD >= 0) closefd(data->cmdFD);
+    if (data->dir) {
+	/* Clean data->dir up if it's still lying around */
+	if (access(data->dir, R_OK) == 0) {
+	    (void) rm_r(ctx, data->dir);
+	}
+	free(data->dir);
+	data->dir = NULL;
+    }
 
-    data->dir = NULL;
     data->fd = -1;
     data->headerFD = -1;
     data->cmdFD = -1;
     data->validatePresent = 0;
     data->filterFailed = 0;
     data->numContentTypeHeaders = 0;
-
     data->dir = strdup_with_log(buffer);
 
     if (!data->dir) {
@@ -868,7 +890,7 @@ envfrom(SMFICTX *ctx, char **from)
     dbuf_init(&dbuf);
 
     /* Generate an ID */
-    append_mx_command(&dbuf, 'i', gen_mx_id(mxid));
+    append_mx_command(&dbuf, 'i', mxid);
 
     /* Write the sender */
     append_mx_command(&dbuf, 'S', from[0]);
@@ -878,27 +900,12 @@ envfrom(SMFICTX *ctx, char **from)
 	append_mx_command(&dbuf, 's', from[i]);
     }
 
-    /* Write some macros */
-    // TODO: Just have one array of macros (regular plus additional) and iterate over it
-    append_macro_value(&dbuf, ctx, "_");
-    append_macro_value(&dbuf, ctx, "auth_authen");
-    append_macro_value(&dbuf, ctx, "auth_author");
-    append_macro_value(&dbuf, ctx, "auth_ssf");
-    append_macro_value(&dbuf, ctx, "auth_type");
-    append_macro_value(&dbuf, ctx, "cert_issuer");
-    append_macro_value(&dbuf, ctx, "cert_subject");
-    append_macro_value(&dbuf, ctx, "cipher");
-    append_macro_value(&dbuf, ctx, "cipher_bits");
-    append_macro_value(&dbuf, ctx, "daemon_name");
-    append_macro_value(&dbuf, ctx, "i");
-    append_macro_value(&dbuf, ctx, "if_addr");
-    append_macro_value(&dbuf, ctx, "if_name");
-    append_macro_value(&dbuf, ctx, "j");
-    append_macro_value(&dbuf, ctx, "mail_addr");
-    append_macro_value(&dbuf, ctx, "mail_host");
-    append_macro_value(&dbuf, ctx, "mail_mailer");
-    append_macro_value(&dbuf, ctx, "tls_version");
-    append_macro_value(&dbuf, ctx, "verify");
+    /* Write the standard macros */
+    macro = StandardSendmailMacros;
+    while (*macro) {
+	append_macro_value(&dbuf, ctx, *macro);
+	macro++;
+    }
 
     /* Write any additional macros requested by user */
     for (i=0; i<NumAdditionalMacros; i++) {
@@ -913,6 +920,7 @@ envfrom(SMFICTX *ctx, char **from)
 
     if (queueid) {
 	append_mx_command(&dbuf, 'Q', queueid);
+	data->qid_written = 1;
     }
 
     /* Write host name and host IP */
@@ -1093,6 +1101,7 @@ rcptto(SMFICTX *ctx, char **to)
 	DEBUG_EXIT("rcptto", "SMFIS_TEMPFAIL");
 	return SMFIS_TEMPFAIL;
     }
+
     rcpt_mailer = smfi_getsymval(ctx, "{rcpt_mailer}");
     if (!rcpt_mailer || !*rcpt_mailer) rcpt_mailer = "?";
 
@@ -1342,6 +1351,18 @@ eoh(SMFICTX *ctx)
 	return SMFIS_TEMPFAIL;
     }
 
+    /* If we don't have a qid yet, try to get one here.
+       Postfix only sets "i" at EOH, DATA and EOM */
+    if (!data->qid || (data->qid == NOQUEUE)) {
+	char *queueid = smfi_getsymval(ctx, "i");
+	if (queueid && *queueid) {
+	    data->qid = strdup_with_log(queueid);
+	    if (!data->qid) {
+		data->qid = NOQUEUE;
+	    }
+	}
+    }
+
     /* We can close headerFD to save a descriptor */
     if (data->headerFD >= 0 && closefd(data->headerFD) < 0) {
 	data->headerFD = -1;
@@ -1399,6 +1420,18 @@ body(SMFICTX *ctx, u_char *text, size_t len)
 	return SMFIS_TEMPFAIL;
     }
 
+    /* If we don't have a qid yet, try to get one here.
+       Postfix only sets "i" at EOH, DATA and EOM */
+    if (!data->qid || (data->qid == NOQUEUE)) {
+	char *queueid = smfi_getsymval(ctx, "i");
+	if (queueid && *queueid) {
+	    data->qid = strdup_with_log(queueid);
+	    if (!data->qid) {
+		data->qid = NOQUEUE;
+	    }
+	}
+    }
+
     /* Write to file and scan body for suspicious characters */
     if (len) {
 	u_char *s = text;
@@ -1502,7 +1535,6 @@ eom(SMFICTX *ctx)
     char chunk[CHUNK];
     char *hdr, *val, *count;
     char *code, *dsn, *reply;
-    char *postfix_queueid;
 
     struct stat statbuf;
     dynamic_buffer dbuf;
@@ -1523,24 +1555,23 @@ eom(SMFICTX *ctx)
 
     dbuf_init(&dbuf);
 
-    /* Postfix doesn't give us a queue-ID until after first
-       RCPT.  If we don't have a queue-ID, see if we can
-       get one now */
-    if (data->qid == NOQUEUE || !(data->qid)) {
-	postfix_queueid = smfi_getsymval(ctx, "i");
-	if (postfix_queueid && *postfix_queueid) {
-	    data->qid = strdup_with_log(postfix_queueid);
+    /* If we don't have a qid yet, try to get one here.
+       Postfix only sets "i" at EOH, DATA and EOM */
+    if (!data->qid || (data->qid == NOQUEUE)) {
+	char *queueid = smfi_getsymval(ctx, "i");
+	if (queueid && *queueid) {
+	    data->qid = strdup_with_log(queueid);
 	    if (!data->qid) {
-		dbuf_free(&dbuf);
 		data->qid = NOQUEUE;
-		cleanup(ctx);
-		DEBUG_EXIT("eom", "SMFIS_TEMPFAIL");
-		return SMFIS_TEMPFAIL;
 	    }
-	    append_mx_command(&dbuf, 'Q', postfix_queueid);
 	}
     }
 
+    if (!data->qid_written && data->qid && (data->qid != NOQUEUE)) {
+	append_mx_command(&dbuf, 'Q', data->qid);
+	data->qid_written = 1;
+    }
+
     /* Signal suspicious body chars */
     if (data->suspiciousBody) {
 	append_mx_command(&dbuf, '?', NULL);
