@@ -8,13 +8,15 @@
 *
 ***********************************************************************/
 
-/* $Id$ */
-
 #ifndef MIMEDEFANG_H
 #define MIMEDEFANG_H 1
 
 #define SMALLBUF 8192
 #define BIGBUF 65536
+
+/* Identifier is 7 chars long: 5 time plus 2 counter */
+#define MX_ID_LEN 7
+
 #include <stddef.h>
 #include <sys/types.h>
 #include <sys/socket.h>   /* For struct sockaddr */
@@ -68,10 +70,14 @@ extern int make_embedded_interpreter(char const *progPath,
 				     char const *subFilter,
 				     int wantStatusReports,
 				     char **env);
+extern void init_embedded_interpreter(int, char **, char **);
+extern void term_embedded_interpreter(void);
 extern void run_embedded_filter(void);
 extern void dump_milter_buildlib_info(void);
 
 #endif
+
+extern char *gen_mx_id(char *);
 /* Magic return values */
 #define MD_TEMPFAIL                    -1
 #define MD_REJECT                       0
